@@ -8,7 +8,7 @@ package Metrics::Any::Adapter::Prometheus;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Carp;
 
@@ -55,17 +55,38 @@ sub new
    }, $class;
 }
 
+sub mangle_name
+{
+   my $self = shift;
+   my ( $name ) = @_;
+
+   $name = join "_", @$name if ref $name eq "ARRAY";
+
+   # TODO: Consider lowercase, squashing unallowed chars to _,...
+
+   return $name;
+}
+
 sub make_counter
 {
    my $self = shift;
    my ( $handle, %args ) = @_;
 
-   my $name = delete $args{name} // $handle;
+   my $name = $self->mangle_name( delete $args{name} // $handle );
    my $help = delete $args{description} // "Metrics::Any counter $handle";
 
+   if( my $units = delete $args{units} ) {
+      # Append _bytes et.al. if required
+      $name .= "_$units" unless $name =~ m/_\Q$units\E$/;
+   }
+   else {
+      # Prometheus policy says unitless counters take _total suffix
+      $name .= "_total";
+   }
+
    $self->{metrics}{$handle} = Net::Prometheus::Registry->register(
       Net::Prometheus::Counter->new(
-         name => "${name}_total",  # Prometheus naming policy
+         name => $name,
          help => $help,
          %args,
       )
@@ -112,7 +133,7 @@ sub make_distribution
    my $self = shift;
    my ( $handle, %args ) = @_;
 
-   my $name  = delete $args{name} // $handle;
+   my $name = $self->mangle_name( delete $args{name} // $handle );
    my $units = delete $args{units};
    my $help  = delete $args{description} // "Metrics::Any $units distribution $handle";
 
@@ -149,7 +170,7 @@ sub make_gauge
    my $self = shift;
    my ( $handle, %args ) = @_;
 
-   my $name = delete $args{name} // $handle;
+   my $name = $self->mangle_name( delete $args{name} // $handle );
    my $help = delete $args{description} // "Metrics::Any gauge $handle";
 
    $self->{metrics}{$handle} = Net::Prometheus::Registry->register(
